==> cbra-0.46.0.tar.gz: cbra-0.46.0/tests/oauth2/servertest.py <==
"""Declares :class:`ServerTest`."""
import datetime
import secrets
import typing

import pytest
import pytest_asyncio
from httpx import AsyncClient
from httpx import Response

import cbra
import ckms
from ckms.jose import PayloadCodec
from cbra.ext.oauth2 import IClient
from cbra.ext.oauth2 import IClientRepository
from cbra.ext.oauth2 import ISubject
from cbra.ext.oauth2 import ISubjectRepository
from cbra.ext.oauth2 import MemoryClientRepository
from cbra.ext.oauth2 import MemoryStorage
from cbra.ext.oauth2 import MemorySubjectRepository
from cbra.ext.oauth2 import OpenAuthorizationServer
from cbra.ext.oauth2 import RFC9068Principal
from cbra.ext.oauth2 import TokenRequestHandler
from cbra.ext.oauth2 import ref
from cbra.lib.auth import IPrincipal


class ServerTest:
    base_url: str = "https://cbra-oauth2.localhost"
    message: str = "Hello world!"

    @pytest.fixture
    def alice(self) -> ISubject:
        return ref.subject.Subject(sub="alice@example.unimatrixone.io")

    @pytest.fixture
    def bob(self) -> ISubject:
        return ref.subject.Subject(sub="bob@example.unimatrixone.io")

    @pytest.fixture
    def trudy(self) -> ISubject:
        """An intruder."""
        return ref.subject.Subject(sub="trudy@example.unimatrixone.io")

    @pytest.fixture
    def current_subject(self, bob: ISubject) -> ISubject:
        return bob

    @pytest_asyncio.fixture
    async def codec(self, current_subject: ISubject) -> PayloadCodec:
        return await self.get_subject_codec(current_subject)

    @pytest.fixture
    def users(self, alice: ISubject, bob: ISubject) -> typing.Dict[str, ISubject]:
        return {
            alice.sub: alice,
            bob.sub: bob
        }

    @pytest_asyncio.fixture
    async def keys(self) -> typing.Dict[str, typing.Dict[str, typing.Any]]:
        return {
            ref.DEFAULT_ENCRYPTION_KEY: {
                'provider': 'local',
                'kty': "OKP",
                'crv': 'Ed448',
                'tags': ['oauth2']
            },
            ref.DEFAULT_SIGNING_KEY: {
                'provider': 'local',
                'kty': "OKP",
                'crv': 'Ed448',
                'tags': ['oauth2']
            },
        }

    @pytest.fixture
    def clients(
        self,
        clients_available: typing.Dict[str, IClient]
    ) -> typing.Generator[typing.Type[IClientRepository], None, None]:
        MemoryClientRepository.configure(clients_available)
        yield MemoryClientRepository
        MemoryClientRepository.clear()

    @pytest.fixture
    def subjects(
        self,
        users: typing.Dict[str, ISubject]
    ) -> typing.Generator[typing.Type[ISubjectRepository], None, None]:
        MemorySubjectRepository.configure(users)
        yield MemorySubjectRepository
        MemorySubjectRepository.clear()

    @pytest.fixture
    def server(
        self,
        keys: typing.Dict[str, typing.Dict[str, typing.Any]],
        clients: typing.Type[IClientRepository],
        subjects: typing.Type[ISubjectRepository]
    ) -> OpenAuthorizationServer:
        MemoryStorage.clear()
        server = OpenAuthorizationServer(
            storage=MemoryStorage,
            clients=clients,
            subjects=subjects,
            token=TokenRequestHandler
        )
        server.register_keys(
            default_encryption_key=ref.DEFAULT_ENCRYPTION_KEY,
            default_signing_key=ref.DEFAULT_SIGNING_KEY,
            keys=keys
        )
        return server

    @pytest.fixture
    def handlers(self) -> typing.List[typing.Any]:
        return []

    @pytest_asyncio.fixture
    async def app(
        self,
        server: OpenAuthorizationServer,
        handlers: typing.List[typing.Any]
    ) -> cbra.Application:
        app = cbra.Application()
        app.add_api_route('/rfc9068', endpoint=self.handle_rfc9068_token)
        server.add_to_app(app)
        for handler in handlers:
            app.add(handler)
        await server.startup()
        return app

    @pytest_asyncio.fixture # type: ignore
    async def client(self, app: cbra.Application):
        params = {
            'app': app,
            'base_url': self.base_url
        }
        async with AsyncClient(**params) as client:
            yield client

    def assert_error(self, response: Response, error: str, status_code: int = 400):
        assert response.status_code == status_code, response.status_code
        assert response.json()['error'] == error, response.json()

    async def handle_rfc9068_token(
        self,
        principal: IPrincipal = RFC9068Principal()
    ):
        assert principal.has_scope("read")
        return {"message": self.message}

==> cbra-0.50.0.tar.gz: cbra-0.50.0/tests/oauth2/servertest.py <==
"""Declares :class:`ServerTest`."""
import typing

import fastapi
import pytest
import pytest_asyncio
from httpx import AsyncClient
from httpx import Response

import cbra
import ckms
from ckms.jose import PayloadCodec
from cbra.ext.jwks import JWKSEndpoint
from cbra.ext.oauth2 import IClient
from cbra.ext.oauth2 import IClientRepository
from cbra.ext.oauth2 import ISubject
from cbra.ext.oauth2 import ISubjectRepository
from cbra.ext.oauth2 import MemoryClientRepository
from cbra.ext.oauth2 import MemoryStorage
from cbra.ext.oauth2 import MemorySubjectRepository
from cbra.ext.oauth2 import OpenAuthorizationServer
from cbra.ext.oauth2 import RFC9068Principal
from cbra.ext.oauth2 import TokenRequestHandler
from cbra.ext.oauth2 import ref
from cbra.lib.auth import IPrincipal


class ServerTest:
    base_url: str = "https://cbra-oauth2.localhost"
    message: str = "Hello world!"

    @pytest.fixture
    def alice(self) -> ISubject:
        return ref.subject.Subject(sub="alice@example.unimatrixone.io")

    @pytest.fixture
    def bob(self) -> ISubject:
        return ref.subject.Subject(sub="bob@example.unimatrixone.io")

    @pytest.fixture
    def trudy(self) -> ISubject:
        """An intruder."""
        return ref.subject.Subject(sub="trudy@example.unimatrixone.io")

    @pytest.fixture
    def current_subject(self, bob: ISubject) -> ISubject:
        return bob

    @pytest_asyncio.fixture
    async def codec(self, current_subject: ISubject) -> PayloadCodec:
        return await self.get_subject_codec(current_subject)

    @pytest.fixture
    def users(self, alice: ISubject, bob: ISubject) -> typing.Dict[str, ISubject]:
        return {
            alice.sub: alice,
            bob.sub: bob
        }

    @pytest_asyncio.fixture
    async def keys(self) -> typing.Dict[str, typing.Dict[str, typing.Any]]:
        return {
            ref.DEFAULT_ENCRYPTION_KEY: {
                'provider': 'local',
                'kty': "OKP",
                'crv': 'Ed448',
                'tags': ['oauth2', 'server']
            },
            ref.DEFAULT_SIGNING_KEY: {
                'provider': 'local',
                'kty': "OKP",
                'crv': 'Ed448',
                'tags': ['oauth2', 'server']
            },
        }

    @pytest.fixture
    def keychain(self, keys: dict[str, typing.Any]) -> ckms.Keychain:
        keychain = ckms.Keychain()
        keychain.register(keys)
        return keychain

    @pytest.fixture
    def clients(
        self,
        clients_available: typing.Dict[str, IClient]
    ) -> typing.Generator[typing.Type[IClientRepository], None, None]:
        MemoryClientRepository.configure(clients_available)
        yield MemoryClientRepository
        MemoryClientRepository.clear()

    @pytest.fixture
    def subjects(
        self,
        users: typing.Dict[str, ISubject]
    ) -> typing.Generator[typing.Type[ISubjectRepository], None, None]:
        MemorySubjectRepository.configure(users)
        yield MemorySubjectRepository
        MemorySubjectRepository.clear()

    @pytest.fixture
    def server(
        self,
        keys: typing.Dict[str, typing.Dict[str, typing.Any]],
        clients: typing.Type[IClientRepository],
        subjects: typing.Type[ISubjectRepository]
    ) -> OpenAuthorizationServer:
        MemoryStorage.clear()
        server = OpenAuthorizationServer(
            storage=MemoryStorage,
            clients=clients,
            subjects=subjects,
            token=TokenRequestHandler,
            default_encryption_key=ref.DEFAULT_ENCRYPTION_KEY,
            default_signing_key=ref.DEFAULT_SIGNING_KEY
        )
        return server

    @pytest.fixture
    def handlers(self) -> typing.List[typing.Any]:
        return []

    @pytest_asyncio.fixture
    async def app(
        self,
        server: OpenAuthorizationServer,
        handlers: typing.List[typing.Any],
        keychain: ckms.Keychain
    ) -> cbra.Application:
        app = cbra.Application(
            keychain=keychain
        )
        app.add_api_route('/rfc9068', endpoint=self.handle_rfc9068_token)
        app.add(JWKSEndpoint)
        server.add_to_app(app)
        for handler_class, base_path in handlers:
            app.add(handler_class, base_path=base_path)
        await app.on_startup()
        await server.startup()
        return app

    @pytest_asyncio.fixture # type: ignore
    async def client(self, app: cbra.Application):
        params = {
            'app': app,
            'base_url': self.base_url
        }
        async with AsyncClient(**params) as client:
            yield client

    def assert_error(self, response: Response, error: str, status_code: int = 400):
        assert response.status_code == status_code, response.status_code
        assert response.json()['error'] == error, response.json()

    async def handle_rfc9068_token(
        self,
        principal: IPrincipal = fastapi.Depends(RFC9068Principal())
    ):
        assert principal.has_scope("read")
        return {"message": self.message}
